/*
 * Copyright (c) 2001 Massachusetts Institute of Technology.
 *
 * This file is part of bgptools. 
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `COPYING' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file AUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the AUTHORS file.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


/*
 *     Nick Feamster (feamster@lcs.mit.edu)
 *     MIT Lab for Computer Science
 *     November 2001
 */


#include "main.h"


void
set_default_opts(struct opts *opt) {


  opt->input_file=NULL;
  opt->input_dir=NULL;

  opt->mrt_output=0;
  opt->long_output=0;
  opt->do_stats=0;
  opt->prefix_tree=0;

  opt->start_time_ep=0;
  opt->end_time_ep=0;

  opt->all_files=0;
  opt->db_out=0;
}

long
make_time(struct tm *tmp, char* datestr) {
  sscanf(datestr, "%d/%d/%d_%d:%d:%d",
	&(tmp->tm_mon), &(tmp->tm_mday), &(tmp->tm_year),
	&(tmp->tm_hour), &(tmp->tm_min), &(tmp->tm_sec));
  tmp->tm_year -= 1900;
  tmp->tm_mon--;
  return mktime(tmp);
}

void
parse_command_line(int argc, char **argv, struct opts *opt) {
  int i;
  char *tok;
  char op;
  
  set_default_opts(opt);

  for (i=1; i<argc; i++) {
    tok = argv[i];

    if (!(strncmp(tok, "-", 1))) {
      op = (char)*(tok+1);
    } else {
      fprintf(stderr, "ERROR: Cannot parse command line!\n");
      exit(0);
    }
    
    switch(op) {
    case 'a':
      opt->start_time_ep = make_time(&(opt->start_time), argv[++i]);
      break;
    case 'b':
      opt->end_time_ep = make_time(&(opt->end_time), argv[++i]);
      break;
    case 'd':
      opt->input_dir = argv[++i];
      if (!opt->input_dir || opt->input_dir[0] == '-') {
	fprintf(stderr, "ERROR: malformed -d option\n");
	exit(0);
      }
      break;
    case 'f':
      opt->input_file = argv[++i];
      if (!opt->input_file || opt->input_file[0] == '-') {
	fprintf(stderr, "ERROR: malformed -f option\n");
	exit(0);
      }
      break;
    case 'l':
      opt->all_files = 1;
      break;
    case 'm':
      opt->mrt_output=1;
      break;
    case 'p':
      opt->prefix_tree = 1;
      opt->dot_file = argv[++i];
      if (!opt->dot_file || opt->dot_file[0] == '-') {
	fprintf(stderr, "ERROR: malformed -p option\n");
      }
      break;
    case 'q':
      opt->db_out=1;
      break;
    case 's':
      opt->do_stats = 1;
      break;
    }

  }

  if (opt->start_time_ep > opt->end_time_ep) {
    fprintf(stderr, "ERROR: Must speficy valid time interval!\n");
    exit(0);
  }
  
}

int is_update_file(const struct dirent *dirent) {
  const char *filename = dirent->d_name;
  return ((strstr(filename, "updates") != NULL) ||
	  (strstr(filename, "bview") != NULL));
}

int is_update_dir(const struct dirent *dirent) {
  const char *filename = dirent->d_name;
  return (((strstr(filename, "0") != NULL) ||
	  (strstr(filename, "1") != NULL)) &&
	  (strstr(filename, "updates") == NULL) &&
	  (strstr(filename, "bview") == NULL));
}

	  

void process_dir_rec(char *dir, struct opts *opt, prefix_tree *p, update_stats *stats, db *up_db) {
  struct dirent **namelist;
  struct stat st;
  char file_path[80];

  unsigned long s_sp = opt->start_time_ep;
  unsigned long s_ep = opt->end_time_ep;

#ifdef BGPDUMP_DEBUG
  fprintf (stderr, "statting from %ld to %ld\n", s_sp, s_ep);
#endif
  
  int n = scandir(dir, &namelist, is_update_dir, alphasort);
  if (n < 0)
    perror("scandir");
  else {
    while(n--) {
      sprintf(file_path, "%s/%s", dir, namelist[n]->d_name);
#ifdef BGPDUMP_DEBUG
      fprintf(stderr, "looking in %s\n", file_path);
#endif
      stat(file_path, &st);
      if (S_ISDIR(st.st_rdev) || 1) {
	process_dir(file_path, opt, p, stats, up_db);
      }
    }  
  }
}

void process_dir(char *dir, struct opts *opt, prefix_tree *p, update_stats *stats, db *up_db) {
  struct dirent **namelist;
  struct stat st;
  char file_path[80];
  char cmd[256];
  int gzip=0;
  char d_name2[80];

  unsigned long s_sp = opt->start_time_ep;
  unsigned long s_ep = opt->end_time_ep;

#ifdef BGPDUMP_DEBUG
  fprintf (stderr, "statting %s from %ld to %ld\n", dir, s_sp, s_ep);
#endif
  
  int n = scandir(dir, &namelist, is_update_file, alphasort);
  if (n < 0)
    perror("scandir");
  else {
    while(n--) {
      sprintf(file_path, "%s/%s", dir, namelist[n]->d_name);
      stat(file_path, &st);
      if ((st.st_mtime > s_sp && st.st_mtime-1000 < s_ep) || opt->all_files) {

	// file is gzippped
	if (strstr(namelist[n]->d_name, ".gz")) {
#ifdef BGPDUMP_DEBUG
	  fprintf(stderr, "gunzipping...\n");
#endif
	  gzip=1;
	  sprintf(cmd, "gunzip %s", file_path);
	  system(cmd);

	  strncpy(d_name2, namelist[n]->d_name, strlen(namelist[n]->d_name));
	  char *p = strstr(d_name2, ".gz");
	  *p = '\0';
	  sprintf(file_path, "%s/%s", dir, d_name2);
	}
	

	
#ifdef BGPDUMP_DEBUG
  	printf("%s %ld\n", file_path, st.st_mtime);
#endif
	process_file(file_path, opt, p, stats, up_db);
	if (gzip) {
	  sprintf(cmd, "gzip %s", file_path);
	  system(cmd);
	}
      }
    }
  }
}

void process_file(char *file, struct opts *opt, prefix_tree *p, update_stats *stats, db *up_db) {

  parse_file_mrt(file, opt, stats);

  /* set by the -q option */
#ifdef __USE_SQLPLUS__
  if (opt->db_out) {
    up_db->add_update_file(file, opt->start_time_ep, opt->end_time_ep);
  }
#endif

  /* set by the -p option */
  if (opt->prefix_tree) {
    p->insert(file);
  }

}


int main(int argc, char **argv) {
  struct opts opt;
  prefix_tree p;
  update_stats stats;
  db *up_db;

  parse_command_line(argc, argv, &opt);

#ifdef __USE_SQLPLUS__
  if (opt.db_out) 
    up_db = new db(DB_USER, DB_HOST);
#endif

  /* set by the -d option, -a option, -b option */
  if ((opt.input_dir && opt.start_time_ep && opt.end_time_ep) ||
      (opt.input_dir && opt.all_files)) 
    process_dir(opt.input_dir, &opt, &p, &stats, up_db);
  else if (opt.input_file)
    process_file(opt.input_file, &opt, &p, &stats, up_db);
  else if (opt.all_files)
    process_dir_rec(opt.input_dir, &opt, &p, &stats, up_db);
  else {
    fprintf(stderr, "ERROR: Must specify input file or dir with time interval!\n");
    exit(0);
  }

  if (opt.do_stats) 
    stats.printStats();

  if (opt.prefix_tree)
    p.print_dot(p.getRoot(), opt.dot_file);
  

}
